@@ -16,7 +16,7 @@ from __future__ import annotations
 
 import os
 import typing
-from typing import Any, Optional  # noqa: H301
+from typing import Any, Optional
 
 from oslo_log import log as logging
 from oslo_service import loopingcall
@@ -161,7 +161,9 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             self,
             connection_properties: dict, hbas) -> list[str]:
         targets = connection_properties['targets']
-        possible_devs = self._get_possible_devices(hbas, targets)
+        addressing_mode = connection_properties.get('addressing_mode')
+        possible_devs = self._get_possible_devices(hbas, targets,
+                                                   addressing_mode)
         host_paths = self._get_host_devices(possible_devs)
         return host_paths
 
@@ -284,7 +286,12 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             if multipath_id:
                 # only set the multipath_id if we found one
                 device_info['multipath_id'] = multipath_id
-
+            if self.device_name:
+                device = os.path.basename(self.device_name)
+                mpath = self._linuxscsi.find_sysfs_multipath_dm(
+                    [device])
+                # Wait for multipath device to be ready for I/O
+                self._linuxscsi.wait_for_mpath_device(mpath)
         else:
             device_path = self.host_device
 
@@ -309,7 +316,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             host_devices.append(host_device)
         return host_devices
 
-    def _get_possible_devices(self, hbas: list, targets: list) -> list:
+    def _get_possible_devices(self, hbas: list, targets: list,
+                              addressing_mode: Optional[str] = None) -> list:
         """Compute the possible fibre channel device options.
 
         :param hbas: available hba devices.
@@ -328,6 +336,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             platform, pci_num = self._get_pci_num(hba)
             if pci_num is not None:
                 for wwn, lun in targets:
+                    lun = self._linuxscsi.lun_for_addressing(lun,
+                                                             addressing_mode)
                     target_wwn = "0x%s" % wwn.lower()
                     raw_devices.append((platform, pci_num, target_wwn, lun))
         return raw_devices
@@ -352,7 +362,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         target_wwn - World Wide Name
         target_lun - LUN id of the volume
         """
-
+        exc = exception.ExceptionChainer()
         devices = []
         wwn = None
 
@@ -368,18 +378,35 @@ class FibreChannelConnector(base.BaseLinuxConnector):
                 wwn = self._linuxscsi.get_scsi_wwn(path)
                 mpath_path = self._linuxscsi.find_multipath_device_path(wwn)
                 if mpath_path:
-                    self._linuxscsi.flush_multipath_device(mpath_path)
+                    with exc.context(force, 'Flushing %s failed', mpath_path):
+                        self._linuxscsi.flush_multipath_device(mpath_path)
             real_path = typing.cast(str, real_path)
             dev_info = self._linuxscsi.get_device_info(real_path)
             devices.append(dev_info)
 
+        # If flush failed, then remove it forcefully since force=True
+        if mpath_path and exc:
+            with exc.context(force, 'Removing multipath %s failed',
+                             mpath_path):
+                mpath_name = os.path.basename(os.path.realpath(mpath_path))
+                self._linuxscsi.multipath_del_map(mpath_name)
+
         LOG.debug("devices to remove = %s", devices)
-        self._remove_devices(connection_properties, devices, device_info)
+        self._remove_devices(connection_properties, devices, device_info,
+                             force, exc)
+
+        if exc:
+            LOG.warning('There were errors removing %s, leftovers may remain '
+                        'in the system', volume_paths)
+            if not ignore_errors:
+                raise exc
 
     def _remove_devices(self,
                         connection_properties: dict,
                         devices: list,
-                        device_info: dict) -> None:
+                        device_info: dict,
+                        force: bool,
+                        exc) -> None:
         # There may have been more than 1 device mounted
         # by the kernel for this volume.  We have to remove
         # all of them
@@ -394,11 +421,12 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         # paths, whereas for multipaths we have multiple link formats.
         was_multipath = '/pci-' not in path_used and was_symlink
         for device in devices:
-            device_path = device['device']
-            flush = self._linuxscsi.requires_flush(device_path,
-                                                   path_used,
-                                                   was_multipath)
-            self._linuxscsi.remove_scsi_device(device_path, flush=flush)
+            with exc.context(force, 'Removing device %s failed', device):
+                device_path = device['device']
+                flush = self._linuxscsi.requires_flush(device_path,
+                                                       path_used,
+                                                       was_multipath)
+                self._linuxscsi.remove_scsi_device(device_path, flush=flush)
 
     def _get_pci_num(self, hba: Optional[dict]) -> tuple:
         # NOTE(walter-boring)
