@@ -702,8 +702,8 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
             force=False, ignore_errors=False,
             device_info=mock.sentinel.device_info)
 
-        get_dev_path_mock.called_once_with(self.CON_PROPS,
-                                           mock.sentinel.device_info)
+        get_dev_path_mock.assert_called_once_with(self.CON_PROPS,
+                                                  mock.sentinel.device_info)
         con_devs_mock.assert_called_once_with(self.CON_PROPS,
                                               mock.sentinel.ips_iqns_luns,
                                               False)
@@ -721,11 +721,12 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
                 mock.Mock(return_value=True))
     @mock.patch.object(iscsi.ISCSIConnector, '_disconnect_connection')
     @mock.patch.object(iscsi.ISCSIConnector, '_get_connection_devices')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_connection',
                        return_value=mock.sentinel.mp_name)
-    def test_cleanup_connection_force_failure(self, remove_mock, flush_mock,
-                                              con_devs_mock, discon_mock):
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_map')
+    def test_cleanup_connection_force_failure(self, remove_map_mock,
+                                              remove_mock, con_devs_mock,
+                                              discon_mock):
 
         # Return an ordered dicts instead of normal dict for discon_mock.assert
         con_devs_mock.return_value = collections.OrderedDict((
@@ -749,7 +750,7 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
             self.CON_PROPS,
             [('ip1:port1', 'tgt1'), ('ip3:port3', 'tgt3')],
             mock.sentinel.force, mock.ANY)
-        flush_mock.assert_called_once_with(mock.sentinel.mp_name)
+        remove_map_mock.assert_called_once_with(mock.sentinel.mp_name)
 
     def test_cleanup_connection_no_data_discoverydb(self):
         self.connector.use_multipath = True
@@ -1748,3 +1749,31 @@ Setting up iSCSI targets: unused
             connection_properties['data'], old_node_startup_values)
         iscsiadm_update_mock.assert_called_once_with(
             recover_connection['data'], "node.startup", "manual")
+
+    @ddt.data(None, 'SAM2')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'lun_for_addressing')
+    @mock.patch.object(iscsi.base_iscsi.BaseISCSIConnector, '_get_all_targets')
+    def test__get_all_targets_no_addressing_mode(self, addressing_mode,
+                                                 get_mock, luns_mock):
+        get_mock.return_value = [
+            (mock.sentinel.portal1, mock.sentinel.iqn1, mock.sentinel.lun1),
+            (mock.sentinel.portal2, mock.sentinel.iqn2, mock.sentinel.lun2)
+        ]
+        luns_mock.side_effect = [mock.sentinel.lun1B, mock.sentinel.lun2B]
+        conn_props = self.CON_PROPS.copy()
+        if addressing_mode:
+            conn_props['addressing_mode'] = addressing_mode
+
+        res = self.connector._get_all_targets(conn_props)
+
+        self.assertEqual(2, luns_mock.call_count)
+        luns_mock.assert_has_calls(
+            [mock.call(mock.sentinel.lun1, addressing_mode),
+             mock.call(mock.sentinel.lun2, addressing_mode)])
+        get_mock.assert_called_once_with(conn_props)
+
+        expected = [
+            (mock.sentinel.portal1, mock.sentinel.iqn1, mock.sentinel.lun1B),
+            (mock.sentinel.portal2, mock.sentinel.iqn2, mock.sentinel.lun2B)
+        ]
+        self.assertListEqual(expected, res)
