@@ -765,11 +765,12 @@ loop0                                     0"""
         self.assertEqual(expected_commands, self.cmds)
 
     def test_multipath_resize_map(self):
-        wwn = '1234567890123456'
-        self.linuxscsi.multipath_resize_map(wwn)
-        expected_commands = ['multipathd resize map %s' % wwn]
+        dm_path = '/dev/dm-5'
+        self.linuxscsi.multipath_resize_map(dm_path)
+        expected_commands = ['multipathd resize map %s' % dm_path]
         self.assertEqual(expected_commands, self.cmds)
 
+    @mock.patch('os_brick.utils.check_valid_device')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
     @mock.patch('os_brick.utils.get_device_size')
@@ -777,7 +778,8 @@ loop0                                     0"""
     def test_extend_volume_no_mpath(self, mock_device_info,
                                     mock_device_size,
                                     mock_scsi_wwn,
-                                    mock_find_mpath_path):
+                                    mock_find_mpath_path,
+                                    mock_valid_dev):
         """Test extending a volume where there is no multipath device."""
         fake_device = {'host': '0',
                        'channel': '0',
@@ -792,6 +794,7 @@ loop0                                     0"""
         wwn = '1234567890123456'
         mock_scsi_wwn.return_value = wwn
         mock_find_mpath_path.return_value = None
+        mock_valid_dev.return_value = True
 
         ret_size = self.linuxscsi.extend_volume(['/dev/fake'])
         self.assertEqual(second_size, ret_size)
@@ -800,14 +803,18 @@ loop0                                     0"""
         expected_cmds = ['tee -a /sys/bus/scsi/drivers/sd/0:0:0:1/rescan']
         self.assertEqual(expected_cmds, self.cmds)
 
+    @mock.patch('os_brick.utils.check_valid_device')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
     @mock.patch('os_brick.utils.get_device_size')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
-    def test_extend_volume_with_mpath(self, mock_device_info,
+    @mock.patch('os.path.realpath')
+    def test_extend_volume_with_mpath(self, mock_realpath,
+                                      mock_device_info,
                                       mock_device_size,
                                       mock_scsi_wwn,
-                                      mock_find_mpath_path):
+                                      mock_find_mpath_path,
+                                      mock_valid_dev):
         """Test extending a volume where there is a multipath device."""
         mock_device_info.side_effect = [{'host': host,
                                          'channel': '0',
@@ -817,8 +824,11 @@ loop0                                     0"""
         mock_device_size.side_effect = [1024, 2048, 1024, 2048, 1024, 2048]
         wwn = '1234567890123456'
         mock_scsi_wwn.return_value = wwn
-        mock_find_mpath_path.return_value = ('/dev/mapper/dm-uuid-mpath-%s' %
-                                             wwn)
+        mpath_path = ('/dev/mapper/dm-uuid-mpath-%s' % wwn)
+        mock_find_mpath_path.return_value = mpath_path
+        dm_path = '/dev/dm-5'
+        mock_realpath.return_value = dm_path
+        mock_valid_dev.return_value = True
 
         ret_size = self.linuxscsi.extend_volume(['/dev/fake1', '/dev/fake2'],
                                                 use_multipath=True)
@@ -828,9 +838,11 @@ loop0                                     0"""
         expected_cmds = ['tee -a /sys/bus/scsi/drivers/sd/0:0:0:1/rescan',
                          'tee -a /sys/bus/scsi/drivers/sd/1:0:0:1/rescan',
                          'multipathd reconfigure',
-                         'multipathd resize map %s' % wwn]
+                         'multipathd resize map %s' % dm_path]
         self.assertEqual(expected_cmds, self.cmds)
+        mock_realpath.assert_called_once_with(mpath_path)
 
+    @mock.patch('os_brick.utils.check_valid_device')
     @mock.patch.object(linuxscsi.LinuxSCSI, '_multipath_resize_map')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
@@ -840,7 +852,8 @@ loop0                                     0"""
                                            mock_device_size,
                                            mock_scsi_wwn,
                                            mock_find_mpath_path,
-                                           mock_mpath_resize_map):
+                                           mock_mpath_resize_map,
+                                           mock_valid_dev):
         """Test extending a volume where there is a multipath device fail."""
         mock_device_info.side_effect = [{'host': host,
                                          'channel': '0',
@@ -855,6 +868,7 @@ loop0                                     0"""
 
         mock_mpath_resize_map.side_effect = putils.ProcessExecutionError(
             stdout="fail")
+        mock_valid_dev.return_value = True
 
         self.assertRaises(
             putils.ProcessExecutionError,
@@ -868,18 +882,22 @@ loop0                                     0"""
                          'multipathd reconfigure']
         self.assertEqual(expected_cmds, self.cmds)
 
+    @mock.patch('os_brick.utils.check_valid_device')
     @mock.patch('time.sleep')
     @mock.patch.object(linuxscsi.LinuxSCSI, '_multipath_resize_map')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
     @mock.patch('os_brick.utils.get_device_size')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
-    def test_extend_volume_with_mpath_pending(self, mock_device_info,
+    @mock.patch('os.path.realpath')
+    def test_extend_volume_with_mpath_pending(self, mock_realpath,
+                                              mock_device_info,
                                               mock_device_size,
                                               mock_scsi_wwn,
                                               mock_find_mpath_path,
                                               mock_mpath_resize_map,
-                                              mock_sleep):
+                                              mock_sleep,
+                                              mock_valid_dev):
         """Test extending a volume where there is a multipath device fail."""
         mock_device_info.side_effect = [{'host': host,
                                          'channel': '0',
@@ -889,11 +907,14 @@ loop0                                     0"""
         mock_device_size.side_effect = [1024, 2048, 1024, 2048, 1024, 2048]
         wwn = '1234567890123456'
         mock_scsi_wwn.return_value = wwn
-        mock_find_mpath_path.return_value = ('/dev/mapper/dm-uuid-mpath-%s' %
-                                             wwn)
+        mpath_path = ('/dev/mapper/dm-uuid-mpath-%s' % wwn)
+        mock_find_mpath_path.return_value = mpath_path
+        dm_path = '/dev/dm-5'
+        mock_realpath.return_value = dm_path
         mock_mpath_resize_map.side_effect = (
             putils.ProcessExecutionError(stdout="timeout"),
             "success")
+        mock_valid_dev.return_value = True
 
         ret_size = self.linuxscsi.extend_volume(['/dev/fake1', '/dev/fake2'],
                                                 use_multipath=True)
@@ -904,8 +925,10 @@ loop0                                     0"""
                          'tee -a /sys/bus/scsi/drivers/sd/1:0:0:1/rescan',
                          'multipathd reconfigure']
         self.assertEqual(expected_cmds, self.cmds)
-        mock_mpath_resize_map.assert_has_calls([mock.call(wwn)] * 2)
+        mock_mpath_resize_map.assert_has_calls([mock.call(dm_path)] * 2)
+        mock_realpath.assert_called_once_with(mpath_path)
 
+    @mock.patch('os_brick.utils.check_valid_device')
     @mock.patch('time.sleep')
     @mock.patch('time.time')
     @mock.patch.object(linuxscsi.LinuxSCSI, '_multipath_resize_map')
@@ -919,7 +942,8 @@ loop0                                     0"""
                                               mock_find_mpath_path,
                                               mock_mpath_resize_map,
                                               mock_currtime,
-                                              mock_sleep):
+                                              mock_sleep,
+                                              mock_valid_dev):
         """Test extending a volume where there is a multipath device fail."""
         mock_device_info.side_effect = [{'host': host,
                                          'channel': '0',
@@ -931,6 +955,7 @@ loop0                                     0"""
         mock_scsi_wwn.return_value = wwn
         mock_find_mpath_path.return_value = ('/dev/mapper/dm-uuid-mpath-%s' %
                                              wwn)
+        mock_valid_dev.return_value = True
 
         # time.time is used to check if our own timeout has been exceeded,
         # which is why it has to be mocked.
@@ -953,6 +978,20 @@ loop0                                     0"""
             ['/dev/fake1', '/dev/fake2'],
             use_multipath=True)
 
+    @mock.patch('os_brick.utils.check_valid_device')
+    def test_extend_volume_with_mpath_path_down(self, mock_valid_dev):
+        """Test extending a volume where there is a path down."""
+        mock_valid_dev.return_value = False
+        dev1 = '/dev/fake1'
+        dev2 = '/dev/fake2'
+
+        self.assertRaises(
+            exception.BrickException,
+            self.linuxscsi.extend_volume,
+            [dev1, dev2],
+            use_multipath=True)
+        mock_valid_dev.assert_called_once_with(self.linuxscsi, dev1)
+
     def test_process_lun_id_list(self):
         lun_list = [2, 255, 88, 370, 5, 256]
         result = self.linuxscsi.process_lun_id(lun_list)
@@ -1299,6 +1338,64 @@ loop0                                     0"""
         self.linuxscsi.multipath_del_path('/dev/sda')
         self.assertEqual(['multipathd del path /dev/sda'], self.cmds)
 
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name', return_value=None)
+    def test_multipath_del_map_not_present(self, name_mock):
+        self.linuxscsi.multipath_del_map('dm-7')
+        self.assertEqual([], self.cmds)
+        name_mock.assert_called_once_with('dm-7')
+
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_execute')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name', return_value=None)
+    def test_multipath_del_map(self, name_mock, exec_mock):
+        exec_mock.side_effect = [putils.ProcessExecutionError, None]
+
+        mpath_name = '3600d0230000000000e13955cc3757800'
+        name_mock.side_effect = [mpath_name, mpath_name, None]
+        self.linuxscsi.multipath_del_map('dm-7')
+
+        self.assertEqual(2, exec_mock.call_count)
+        exec_mock.assert_has_calls(
+            [mock.call('multipathd', 'del', 'map', mpath_name,
+                       run_as_root=True, timeout=5,
+                       root_helper=self.linuxscsi._root_helper)] * 2)
+        self.assertEqual(3, name_mock.call_count)
+        name_mock.assert_has_calls([mock.call('dm-7')] * 3)
+
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_execute')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
+    def test_multipath_del_map_retries_cmd_fails(self, name_mock, exec_mock):
+        exec_mock.side_effect = putils.ProcessExecutionError
+        mpath_name = '3600d0230000000000e13955cc3757800'
+        name_mock.return_value = mpath_name
+        self.assertRaises(putils.ProcessExecutionError,
+                          self.linuxscsi.multipath_del_map, 'dm-7')
+
+        self.assertEqual(3, exec_mock.call_count)
+        exec_mock.assert_has_calls(
+            [mock.call('multipathd', 'del', 'map', mpath_name,
+                       run_as_root=True, timeout=5,
+                       root_helper=self.linuxscsi._root_helper)] * 3)
+
+        self.assertEqual(3, name_mock.call_count)
+        name_mock.assert_has_calls([mock.call('dm-7')] * 3)
+
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_execute')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
+    def test_multipath_del_map_retries_remains(self, name_mock, exec_mock):
+        mpath_name = '3600d0230000000000e13955cc3757800'
+        name_mock.return_value = mpath_name
+        self.assertRaises(exception.BrickException,
+                          self.linuxscsi.multipath_del_map, 'dm-7')
+
+        self.assertEqual(3, exec_mock.call_count)
+        exec_mock.assert_has_calls(
+            [mock.call('multipathd', 'del', 'map', mpath_name,
+                       run_as_root=True, timeout=5,
+                       root_helper=self.linuxscsi._root_helper)] * 3)
+
+        self.assertEqual(6, name_mock.call_count)
+        name_mock.assert_has_calls([mock.call('dm-7')] * 6)
+
     @ddt.data(('/dev/sda', '/dev/sda', False, True, None),
               # This checks that we ignore the was_multipath parameter if it
               # doesn't make sense (because the used path is the one we are
@@ -1322,3 +1419,63 @@ loop0                                     0"""
             if real_paths:
                 mocked.assert_has_calls([mock.call(path),
                                          mock.call(path_used)])
+
+    @ddt.data(None, 'SAM', 'transparent')
+    def test_lun_for_addressing_transparent_sam(self, mode):
+        lun = self.linuxscsi.lun_for_addressing(1, mode)
+        self.assertEqual(1, lun)
+        lun = self.linuxscsi.lun_for_addressing(256, mode)
+        self.assertEqual(256, lun)
+
+    @ddt.data(1, 'SAM3', 'TRANSPARENT', 'sam', 'sam2')
+    def test_lun_for_addressing_bad(self, mode):
+        self.assertRaises(exception.InvalidParameterValue,
+                          self.linuxscsi.lun_for_addressing, 1, mode)
+
+    @ddt.data((1, 1), (100, 100), (256, 16640), (1010, 17394))
+    @ddt.unpack
+    def test_lun_for_addressing_sam2(self, original_lun, expected_lun):
+        lun = self.linuxscsi.lun_for_addressing(original_lun, 'SAM2')
+        self.assertEqual(expected_lun, lun)
+
+    @ddt.data((0, 16384), (100, 16484), (256, 16640), (1010, 17394))
+    @ddt.unpack
+    def test_lun_for_addressing_sam3_flat(self, original_lun, expected_lun):
+        lun = self.linuxscsi.lun_for_addressing(original_lun, 'SAM3-flat')
+        self.assertEqual(expected_lun, lun)
+
+    @mock.patch.object(linuxscsi, 'LOG')
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_execute')
+    def test_wait_for_mpath_device(self, exec_mock, mock_log):
+        exec_mock.return_value = (
+            "3293379.070675 | /dev/dm-7: path sdb is usable",
+            None,
+        )
+        mpath_name = 'dm-7'
+        self.linuxscsi.wait_for_mpath_device(mpath_name)
+
+        self.assertEqual(1, exec_mock.call_count)
+        exec_mock.assert_called_once_with(
+            'multipath', '-C', mpath_name,
+            attempts=4, interval=1,
+            run_as_root=True,
+            root_helper=self.linuxscsi._root_helper)
+        mock_log.error.assert_not_called()
+
+    @mock.patch.object(linuxscsi, 'LOG')
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_execute')
+    def test_wait_for_mpath_device_fails(self, exec_mock, mock_log):
+        exec_mock.side_effect = putils.ProcessExecutionError
+
+        mpath_name = 'dm-7'
+        exc = self.assertRaises(putils.ProcessExecutionError,
+                                self.linuxscsi.wait_for_mpath_device, 'dm-7')
+
+        exec_mock.assert_called_once_with(
+            'multipath', '-C', mpath_name,
+            attempts=4, interval=1,
+            run_as_root=True,
+            root_helper=self.linuxscsi._root_helper)
+        mock_log.error.assert_called_once_with(
+            "Failed to get mpath device %(mpath)s ready for "
+            "I/O: %(except)s", {'mpath': mpath_name, 'except': exc})
